@@ -575,12 +575,12 @@ def pScope(pf):
 
 
 def pExists(sid):  # @Match(A)
-    return lambda px: px.getstate(px.state, sid) != None
+    return lambda px: getstate(px.state, sid) != None
 
 
 def pMatch(sid):  # @Match(A)
     def match(px):
-        state = px.getstate(px.state, sid)
+        state = getstate(px.state, sid)
         if state is not None and px.inputs.startswith(state.val, px.pos):
             px.pos += len(state.val)
             return True
@@ -817,7 +817,7 @@ class ParseTree(list):
         mark = ''.join(mark)
         return (self.urn_, spos, linenum, column, line, mark)
 
-    def showing(self, msg='Syntax Error'):
+    def message(self, msg='Syntax Error'):
         urn, pos, linenum, cols, line, mark = self.decode()
         return '{} ({}:{}:{}+{})\n{}\n{}'.format(msg, urn, linenum, cols, pos, line, mark)
 
@@ -827,46 +827,88 @@ class ParseTree(list):
     def isSyntaxError(self):
         return self.tag_ == 'err'
 
+    def keys(self):
+        ks = []
+        for key in self.__dict__:
+            v = self.__dict__[key]
+            if isinstance(v, ParseTree):
+                ks.append(v)
+        return ks
+
+    def subs(self):
+        es = []
+        for i, child in enumerate(self):
+            es.append((child.spos_, '', child))
+        for key in self.__dict__:
+            v = self.__dict__[key]
+            if isinstance(v, ParseTree):
+                es.append((v.spos_, key, v))
+        es.sort()
+        return [(x[1], x[2]) for x in es]
+
+    def get(self, key):
+        return getattr(self, key)
+
+    def set(self, key, t):
+        assert isinstance(t, ParseTree)
+        if key == '':
+            self.append(key, t)
+        else:
+            setattr(self, key, t)
+
     def __str__(self):
         s = self.inputs_[self.spos_:self.epos_]
         return s.decode('utf-8') if isinstance(s, bytes) else s
 
     def __repr__(self):
         if self.isSyntaxError():
-            return self.showing('Syntax Error')
+            return self.message('Syntax Error')
         sb = []
         self.strOut(sb)
         return "".join(sb)
 
-    def dump(self, indent='\n  ', tab='  ', tag=nop, edge=nop, token=nop):
+    def dump(self, indent='\n', tab='  ', tag=nop, edge=nop, token=nop):
         if self.isSyntaxError():
-            print(self.showing('Syntax Error'))
+            print(self.message('Syntax Error'))
         else:
             sb = []
-            self.strOut(sb)
+            self.strOut(sb, indent, tab, '', tag, edge, token)
             print("".join(sb))
 
-    def strOut(self, sb, indent='\n  ', tab='  ', tag=nop, edge=nop, token=nop):
-        sb.append("[" + tag(f'#{self.tag_}'))
-        hasContent = False
-        next_indent = indent + tab
-        for child in self:
-            hasContent = True
-            sb.append(indent)
-            if hasattr(child, 'strOut'):
-                child.strOut(sb, next_indent, tab, tag, edge, token)
-            else:
-                sb.append(repr(child))
-        for key in self.__dict__:
-            v = self.__dict__[key]
-            if isinstance(v, ParseTree):
-                hasContent = True
-                sb.append(indent)
-                sb.append(edge(key) + ': ')
-                v.strOut(sb, next_indent, tab, tag, edge, token)
-        if not hasContent:
+    def strOut(self, sb, indent='\n  ', tab='  ', prefix='', tag=nop, edge=nop, token=nop):
+        sb.append(indent + prefix + "[" + tag(f'#{self.tag_}'))
+        subs = self.subs()
+        if len(subs) > 0:
+            next_indent = indent + tab
+            for label, child in subs:
+                prefix = edge(label) + ': ' if label != '' else ''
+                child.strOut(sb, next_indent, tab, prefix, tag, edge, token)
+            sb.append(indent + "]")
+        else:
             sb.append(' ' + token(repr(str(self))))
-        sb.append("]")
+            sb.append("]")
+
+    # def strOut2(self, sb, indent='\n  ', tab='  ', tag=nop, edge=nop, token=nop):
+    #     sb.append("[" + tag(f'#{self.tag_}'))
+    #     hasContent = False
+    #     next_indent = indent + tab
+    #     for child in self:
+    #         hasContent = True
+    #         sb.append(indent)
+    #         if hasattr(child, 'strOut'):
+    #             child.strOut(sb, next_indent, tab, tag, edge, token)
+    #         else:
+    #             sb.append(repr(child))
+    #     for key in self.__dict__:
+    #         v = self.__dict__[key]
+    #         if isinstance(v, ParseTree):
+    #             hasContent = True
+    #             sb.append(indent)
+    #             sb.append(edge(key) + ': ')
+    #             v.strOut(sb, next_indent, tab, tag, edge, token)
+    #     if not hasContent:
+    #         sb.append(' ' + token(repr(str(self))))
+    #     sb.append("]")
 
 
 def PTree2ParseTree(pt: PTree, urn, inputs):
